#------------------------------------------------------------
"""

 CopyRightBodyToLeft.py

    This does two separate things:

    1. On centerline parts [Neck,Chest,Abdomen,Hip] it copies values of MTs
    with left names to corresponding right MTs.  So, if Neck has Goiter_L and Goiter_R
    morphs, Goiter_L's value will be copied to Goiter_R.

    2. On paired parts [Left Collar / Right Collar, etc.] it copies MTs on
    the left part to the right part.  So if Left Shoulder has a SuperNerd morph,
    and Right Shoulder also has a SuperNerd morph, the left value will be
    copied to the right shoulder.

    Note on naming:  We try all four possible switchovers.  Ideally this
    should be only L/R and LEFT/RIGHT, but some commercial figures do
    have asymmetrical names, like "Brow-Angry-Right" and "Brow-Angry-L"
    on Koshini.

 Copyright (c) 2005 David G. Drumright (ockham)

"""

#------------------------------------------------------------

import poser
import string
scene = poser.Scene()

#-----------------------------------------------------------------------------
def ReplaceRbyL(Given,IncludeSame):
   UpperGiven=string.upper(Given)
   ReturnList=[]
   if IncludeSame:  ReturnList.append(Given)

   # Trying a different approach.  Generate all possible switcheroos, and
   # try them.  The args are the original morph name and an IncludeSame flag.
   # IncludeSame should only be TRUE when using an opposite-side part, where the
   # most likely case is that the swapper is same MT name.
   # On a midline part, set it FALSE so we don't just hit the original MT.

   # If the given name has LEFT anywhere in it, we give back a list
   # consisting of (1) the same name (2) the name with LEFT replaced by RIGHT
   # and (3) the name with LEFT replaced by R.

   # If no LEFT, we check for any L.

   # If the given name has L anywhere in it, we give back a list
   # consisting of (1) the same name (2) the name with L replaced by RIGHT
   # and (3) the name with L replaced by R.

   # Most of these replacements will be useless; it's up to the calling routine
   # to check their existence.

   # Look for left:
   if string.count(UpperGiven,'RIGHT'):
       # Really only three possibilities here.
       if string.count(Given,'Right'):
           ReturnList.append(string.replace(Given,'Right','Left'))
           ReturnList.append(string.replace(Given,'Right','L'))
       elif string.count(Given,'RIGHT'):
           ReturnList.append(string.replace(Given,'RIGHT','LEFT'))
           ReturnList.append(string.replace(Given,'RIGHT','L'))
       elif string.count(Given,'right'):
           ReturnList.append(string.replace(Given,'right','left'))
           ReturnList.append(string.replace(Given,'right','l'))
   elif string.count(UpperGiven,'R'):
       # Replace each occurrence separately. Most of these will be stupid.
       for i in range(len(Given)):
           if Given[i]=='r':
               NewGiven=Given[:i]+'l'+Given[(i+1):] # make a copy
               ReturnList.append(NewGiven)
           if Given[i]=='R':
               NewGiven=Given[:i]+'L'+Given[(i+1):] # make a copy
               ReturnList.append(NewGiven)

   return ReturnList

#-----------------------------------------------------------------------------

BaseFig = scene.CurrentFigure() # should have been set
if not BaseFig: exit

# Begin centerline loop
CenterlinePartsUC = ["BODY","NECK","CHEST","ABDOMEN","HIP"]
for OneBodyPart in BaseFig.Actors():
    OneBodyPartUC = string.strip(string.upper(OneBodyPart.Name()))
    if OneBodyPartUC not in CenterlinePartsUC: continue

    # Look for MTs that have same name except for Right/Left

    for OrigParm in OneBodyPart.Parameters():
        if OrigParm.IsMorphTarget():
            TrySwaps = ReplaceRbyL(OrigParm.Name(),0) # Get list of replacements, excluding same name
            SwapParm=None
            for SwapName in TrySwaps:
                try:
                    SwapParm=OneBodyPart.Parameter(SwapName)
                except:
                    continue
            if SwapParm:
                Lval = OrigParm.Value()
                SwapParm.SetValue(Lval)

#------------------------------------------------------------

# Begin side-parts loop.  In this case we do the L/R switch
# on body parts and use same-name parameters.

SwapPart=None
for OrigPart in BaseFig.Actors():
    if not OrigPart.IsBodyPart(): continue
    OPLC=OrigPart.Name()
    OPUC=string.upper(OPLC)
    if OPUC not in CenterlinePartsUC:
        TrySwaps=ReplaceRbyL(OPLC,0)
        if not TrySwaps: continue  # This isn't a Left side part.
        for OneSwap in TrySwaps:
            try:
                SwapPart=BaseFig.Actor(OneSwap)
            except:
                continue
    if SwapPart: # We have an opposing body part
        for OrigParm in OrigPart.Parameters():
            TrySwaps=ReplaceRbyL(OrigParm.Name(),1) # Can have same name here
            if not TrySwaps: continue
            SwapParm=None
            for OneSwap in TrySwaps:
                try:
                    SwapParm=SwapPart.Parameter(OneSwap)
                except:
                    continue
            if not SwapParm: continue
            Lval = OrigParm.Value()
            Rval = SwapParm.Value()
            SwapParm.SetValue(Lval)
            #OrigParm.SetValue(Rval)

#------------------------------------------------------------
