#------------------------------------------------------------
"""

 SwapFaceMorphs.py

    Copies values of MTs with left names to corresponding right MTs. So,
    if Head has Smile_L and Smile_R morphs, or LeftFrown and RightFrown,
    the Left value will be copied to the Right.

    Note on naming:  We try all four possible switchovers.  Ideally this
    should be only L/R and LEFT/RIGHT, but some commercial figures do
    have asymmetrical names, like "Brow-Angry-Right" and "Brow-Angry-L"
    on Koshini.

 Copyright (c) 2005 David G. Drumright (ockham)

"""

#------------------------------------------------------------

import poser
import string
scene = poser.Scene()

#-----------------------------------------------------------------------------
def ReplaceLbyR(Given,IncludeSame):
   UpperGiven=string.upper(Given)
   ReturnList=[]
   if IncludeSame:  ReturnList.append(Given)

   # Trying a different approach.  Generate all possible switcheroos, and
   # try them.  The args are the original morph name and an IncludeSame flag.
   # IncludeSame should only be TRUE when using an opposite-side part, where the
   # most likely case is that the swapper is same MT name.
   # On a midline part, set it FALSE so we don't just hit the original MT.

   # If the given name has LEFT anywhere in it, we give back a list
   # consisting of (1) the same name (2) the name with LEFT replaced by RIGHT
   # and (3) the name with LEFT replaced by R.

   # If no LEFT, we check for any L.

   # If the given name has L anywhere in it, we give back a list
   # consisting of (1) the same name (2) the name with L replaced by RIGHT
   # and (3) the name with L replaced by R.

   # Most of these replacements will be useless; it's up to the calling routine
   # to check their existence.

   # Look for left:
   if string.count(UpperGiven,'LEFT'):
       # Really only three possibilities here.
       if string.count(Given,'Left'):
           ReturnList.append(string.replace(Given,'Left','Right'))
           ReturnList.append(string.replace(Given,'Left','R'))
       elif string.count(Given,'LEFT'):
           ReturnList.append(string.replace(Given,'LEFT','RIGHT'))
           ReturnList.append(string.replace(Given,'LEFT','R'))
       elif string.count(Given,'left'):
           ReturnList.append(string.replace(Given,'left','right'))
           ReturnList.append(string.replace(Given,'left','r'))
   elif string.count(UpperGiven,'L'):
       # Replace each occurrence separately. Most of these will be stupid.
       for i in range(len(Given)):
           if Given[i]=='l':
               NewGiven=Given[:i]+'r'+Given[(i+1):] # make a copy
               ReturnList.append(NewGiven)
           if Given[i]=='L':
               NewGiven=Given[:i]+'R'+Given[(i+1):] # make a copy
               ReturnList.append(NewGiven)

   return ReturnList
#-----------------------------------------------------------------------------
BaseFig = scene.CurrentFigure() # should have been set
if not BaseFig: exit

try:
    OneActor = BaseFig.Actor("Head")
except:
    raise "No head!"

# Look for MTs that have same name except for Right/Left
for OrigParm in OneActor.Parameters():
    if not OrigParm.IsMorphTarget(): continue
    SwapNames=ReplaceLbyR(OrigParm.Name(),0)
    if not SwapNames: continue
    SwapParm=None
    for SwapName in SwapNames:
        try:
            SwapParm=OneActor.Parameter(SwapName)
        except:
            continue
        if not SwapParm: continue
        Lval = OrigParm.Value()
        Rval = SwapParm.Value()
        SwapParm.SetValue(Lval)
        OrigParm.SetValue(Rval)

#------------------------------------------------------------
